@@ -195,19 +195,26 @@ NextBestThreePlusThree <- function()
 ##' \code{maxOverdoseProb} of having an overdose toxicity, as specified by the
 ##' overdose interval \code{overdose}. Then, it picks under the remaining
 ##' admissible doses the one that maximizes the probability to be in the
-##' \code{target} biomarker range, relative to the maximum biomarker level
+##' \code{target} biomarker range, by default relative to the maximum biomarker level
 ##' across the dose grid or relative to the Emax parameter in case a parametric
 ##' model was selected (e.g. \code{\linkS4class{DualEndpointBeta}},
-##' \code{\linkS4class{DualEndpointEmax}}))
+##' \code{\linkS4class{DualEndpointEmax}})) However, is \code{scale} is set to
+##' "absolute" then the natural absolute biomarker scale can be used to set a target.
 ##'
-##' @slot target the biomarker target range, relative to the maximum, that
-##' needs to be reached. For example, (0.8, 1.0) means we target a dose
+##' @slot target the biomarker target range, that
+##' needs to be reached. For example, (0.8, 1.0) and \code{scale="relative"} 
+##' means we target a dose
 ##' with at least 80\% of maximum biomarker level. As an other example,
 ##' (0.5, 0.8) would mean that we target a dose between 50\% and 80\% of
 ##' the maximum biomarker level.
+##' @slot scale either \code{relative} (default, then the \code{target} is interpreted 
+##' relative to the maximum, so must be a probability range) or \code{absolute}
+##' (then the \code{target} is interpreted as absolute biomarker range)
 ##' @slot overdose the overdose toxicity interval (lower limit excluded, upper
 ##' limit included)
 ##' @slot maxOverdoseProb maximum overdose probability that is allowed
+##' @slot targetThresh which target probability threshold needs to be fulfilled before the 
+##' target probability will be used for deriving the next best dose (default: 0.01)
 ##' 
 ##' @example examples/Rules-class-NextBestDualEndpoint.R
 ##' @export
@@ -215,22 +222,36 @@ NextBestThreePlusThree <- function()
 .NextBestDualEndpoint <-
     setClass(Class="NextBestDualEndpoint",
              representation(target="numeric",
+                            scale="character",
                             overdose="numeric",
-                            maxOverdoseProb="numeric"),
+                            maxOverdoseProb="numeric",
+                            targetThresh="numeric"),
              prototype(target=c(0.9,1),
+                       scale="relative",
                        overdose=c(0.35, 1),
-                       maxOverdoseProb=0.25),
+                       maxOverdoseProb=0.25,
+                       targetThresh=0.01),
              contains=list("NextBest"),
              validity=
                  function(object){
                      o <- Validate()
 
-                     o$check(is.probRange(object@target),
-                             "target has to be a probability range")
+                     o$check(is.scalar(object@scale) && object@scale %in% c("relative", "absolute"),
+                             "scale must be either 'relative' or 'absolute'")
+                     if(object@scale == "relative")
+                     {
+                       o$check(is.probRange(object@target),
+                               "target has to be a probability range when scale='relative'")
+                     } else {
+                       o$check(is.range(object@target),
+                               "target must be a numeric range")
+                     }
                      o$check(is.probRange(object@overdose),
                              "overdose has to be a probability range")
                      o$check(is.probability(object@maxOverdoseProb),
                              "maxOverdoseProb has to be a probability")
+                     o$check(is.probability(object@targetThresh),
+                             "targetThresh has to be a probability")
 
                      o$result()
                  })
@@ -239,19 +260,26 @@ validObject(.NextBestDualEndpoint())
 ##' Initialization function for "NextBestDualEndpoint"
 ##'
 ##' @param target see \code{\linkS4class{NextBestDualEndpoint}}
+##' @param scale see \code{\linkS4class{NextBestDualEndpoint}}
 ##' @param overdose see \code{\linkS4class{NextBestDualEndpoint}}
 ##' @param maxOverdoseProb see \code{\linkS4class{NextBestDualEndpoint}}
+##' @param targetThresh see \code{\linkS4class{NextBestDualEndpoint}}
 ##' @return the \code{\linkS4class{NextBestDualEndpoint}} object
 ##'
 ##' @export
 ##' @keywords methods
 NextBestDualEndpoint <- function(target,
+                                 scale=c("relative", "absolute"),
                                  overdose,
-                                 maxOverdoseProb)
+                                 maxOverdoseProb,
+                                 targetThresh=0.01)
 {
-    .NextBestDualEndpoint(target=target,
-                          overdose=overdose,
-                          maxOverdoseProb=maxOverdoseProb)
+  scale <- match.arg(scale)
+  .NextBestDualEndpoint(target=target,
+                        scale=scale,
+                        overdose=overdose,
+                        maxOverdoseProb=maxOverdoseProb,
+                        targetThresh=targetThresh)
 }
 
 
@@ -782,8 +810,12 @@ StoppingMTDdistribution <- function(target,
 
 ##' Stop based on probability of target biomarker
 ##'
-##' @slot target the biomarker target range, relative to the maximum, that
-##' needs to be reached. So this must be a probability range (1 is allowed here)
+##' @slot target the biomarker target range, that
+##' needs to be reached. For example, (0.8, 1.0) and \code{scale="relative"} 
+##' means we target a dose with at least 80\% of maximum biomarker level. 
+##' @slot scale either \code{relative} (default, then the \code{target} is interpreted 
+##' relative to the maximum, so must be a probability range) or \code{absolute}
+##' (then the \code{target} is interpreted as absolute biomarker range)
 ##' @slot prob required target probability for reaching sufficient precision
 ##' 
 ##' @example examples/Rules-class-StoppingTargetBiomarker.R
@@ -792,16 +824,26 @@ StoppingMTDdistribution <- function(target,
 .StoppingTargetBiomarker <-
     setClass(Class="StoppingTargetBiomarker",
              representation(target="numeric",
+                            scale="character",
                             prob="numeric"),
              prototype(target=c(0.9, 1),
+                       scale="relative",
                        prob=0.3),
              contains="Stopping",
              validity=
                  function(object){
                      o <- Validate()
-
-                     o$check(is.probRange(object@target),
-                             "target has to be a probability range")
+                     
+                     o$check(is.scalar(object@scale) && object@scale %in% c("relative", "absolute"),
+                             "scale must be either 'relative' or 'absolute'")
+                     if(object@scale == "relative")
+                     {
+                       o$check(is.probRange(object@target),
+                               "target has to be a probability range when scale='relative'")
+                     } else {
+                       o$check(is.range(object@target),
+                               "target must be a numeric range")
+                     }
                      o$check(is.probability(object@prob,
                                             bounds=FALSE),
                              "prob must be probability > 0 and < 1")
@@ -814,15 +856,19 @@ validObject(.StoppingTargetBiomarker())
 ##' Initialization function for "StoppingTargetBiomarker"
 ##'
 ##' @param target see \code{\linkS4class{StoppingTargetBiomarker}}
+##' @param scale see \code{\linkS4class{StoppingTargetBiomarker}}
 ##' @param prob see \code{\linkS4class{StoppingTargetBiomarker}}
 ##' @return the \code{\linkS4class{StoppingTargetBiomarker}} object
 ##'
 ##' @export
 ##' @keywords methods
 StoppingTargetBiomarker <- function(target,
+                                    scale=c("relative", "absolute"),
                                     prob)
 {
+  scale <- match.arg(scale)
     .StoppingTargetBiomarker(target=target,
+                             scale=scale,
                              prob=prob)
 }
 
@@ -1163,7 +1209,7 @@ setClass(Class="CohortSize",
                      o$check(identical(length(object@cohortSize),
                                        length(object@intervals)),
                              "cohortSize must have same length as intervals")
-                     o$check(all(object@cohortSize > 0),
+                     o$check(all(object@cohortSize >= 0),
                              "cohortSize must only contain positive integers")
                      o$check(! is.unsorted(object@intervals, strictly=TRUE),
                              "intervals has to be sorted and have unique values")
@@ -1215,7 +1261,7 @@ CohortSizeRange <- function(intervals,
                      o$check(identical(length(object@cohortSize),
                                        length(object@DLTintervals)),
                              "cohortSize must have same length as DLTintervals")
-                     o$check(all(object@cohortSize > 0),
+                     o$check(all(object@cohortSize >= 0),
                              "cohortSize must only contain positive integers")
                      o$check(! is.unsorted(object@DLTintervals, strictly=TRUE),
                              "DLTintervals has to be sorted and have unique values")
@@ -1264,7 +1310,7 @@ CohortSizeDLT <- function(DLTintervals,
                  function(object){
                      o <- Validate()
 
-                     o$check(is.scalar(object@size) && (object@size > 0),
+                     o$check(is.scalar(object@size) && (object@size >= 0),
                              "size needs to be positive scalar")
 
                      o$result()
